# Copyright (c) SenseTime. All Rights Reserved.

from __future__ import absolute_import 
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import torch.nn as nn
import torch

class AdjustLayer(nn.Module):
    def __init__(self, inplane, outplane):
        super(AdjustLayer, self).__init__()
        self.downsample = nn.Sequential(
                nn.Conv2d(inplane, outplane, kernel_size=1, bias=False),
                nn.BatchNorm2d(outplane),
                )

    def forward(self, x):
        x = self.downsample(x)
        if x.size(3) < 20:
            l = 4
            r = l + 7
            x = x[:, :, l:r, l:r]
        return x

class AdjustAllLayer(nn.Module):
    def __init__(self, in_channels=(128*4, 256*4, 512*4), out_channels=(256, 256, 256)):
        super(AdjustAllLayer, self).__init__()
        self.num = len(out_channels)
        if self.num == 1:
            self.downsample = AdjustLayer(in_channels[0], out_channels[0])
        else:
            for i in range(self.num):
                self.add_module('downsample'+str(i+2),
                        AdjustLayer(in_channels[i], out_channels[i]))

    def forward(self, features):
        if self.num == 1:
            return self.downsample(features)
        else:
            out = []
            for i in range(self.num):
                adj_layer = getattr(self, 'downsample'+str(i+2))
                out.append(adj_layer(features[i]))
            return out
if __name__ == '__main__':
    net = AdjustAllLayer()
    # print(net)
    net = net.cuda()

    # checkpoint = torch.load('/home/lyc/Documents/VOT2019/experiments/siamrpn_r50_l234_dwxcorr_lt/model.pth')
    # net.load_state_dict(checkpoint)

    var = [torch.randn(1,512,15,15).float().cuda(), torch.randn(1,1024,15,15).float().cuda(), torch.randn(1,2048,15,15).float().cuda()]
    # var = Variable(var)

    A1,a2,a3 = net(var)
    print('*************')
    var = torch.FloatTensor(1,512,31,31).cuda()
    # var = Variable(var)

    B = net(var)