# from IPython import embed

import numpy as np
import pickle
import os
import cv2
import functools
import xml.etree.ElementTree as ET
import sys
import multiprocessing as mp

from multiprocessing import Pool
from fire import Fire
from tqdm import tqdm
from glob import glob
import pandas as pd

sys.path.append(os.getcwd())
from config import config
from utils.utils import get_instance_image, add_box_img


def worker(output_dir, video_dir):
    instance_crop_size = 500
    # video_dir = os.path.join(video_dir, 'color')
    image_names = glob(os.path.join(video_dir, 'color/*.jpg'))
    image_names = sorted(image_names, key=lambda x: int(x.split('/')[-1].split('.')[0]))
    video_name = video_dir.split('/')[-1]
    save_folder = os.path.join(output_dir, video_name)
    if not os.path.exists(save_folder):
        os.mkdir(save_folder)
    trajs = {}
    for i, image_name in enumerate(image_names):
        img = cv2.imread(image_name)
        h_img, w_img, _ = img.shape
        img_mean = tuple(map(int, img.mean(axis=(0, 1))))
        # anno_name = image_name.replace(image_name.split('/')[-1], 'groundtruth.txt')
        gt_bboxes = pd.read_csv(os.path.join(video_dir, "groundtruth.txt"), sep='\t|,| ',
                                header=None, names=['xmin', 'ymin', 'width', 'height'],
                                engine='python')
        filename = os.path.splitext(os.path.basename(image_name))[0]
        if 0 in trajs:
            trajs[0].append(filename)
        else:
            trajs[0] = [filename]
        bboxes = gt_bboxes.iloc[i].values
        bbox = np.array(
            [bboxes[0] + bboxes[2] / 2 - 0.5, bboxes[1] + bboxes[3] / 2 - 0.5, bboxes[2],
             bboxes[3]])
        if (np.array(bbox) != np.array(bbox)).any():  # 不为nan
            pass
        else:
            instance_img, w, h, _ = get_instance_image(img, bbox,
                                                       config.exemplar_size, instance_crop_size,
                                                       config.context_amount,
                                                       img_mean)
            instance_img_name = os.path.join(save_folder,
                                             filename + ".{:02d}.x_{:.2f}_{:.2f}_{:.0f}_{:.0f}.jpg".format(0, w,
                                                                                                           h, w_img,
                                                                                                           h_img))
            cv2.imwrite(instance_img_name, instance_img)
    return video_name, trajs


# def processing(vid_dir, ytb_dir, got_dir, coco_dir, output_dir, num_threads=mp.cpu_count()):
#     # get all 4417 videos in vid and all video in ytbb
#     vid_video_dir = os.path.join(vid_dir, 'Data/VID')
#     ytb_video_dir = ytb_dir
#     got_video_dir = os.path.join(got_dir, 'full_data/train_data')
#     coco_dir = os.path.join(coco_dir, 'Data')
def processing(vot_dir, output_dir, num_threads=mp.cpu_count()):

    all_videos = glob(os.path.join(vot_dir, '*'))
    meta_data = []
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)
    all_videos = [x for x in all_videos if 'list.txt' not in x and 'zip' not in x]

    # for video in tqdm(all_videos):
    #     functools.partial(worker, output_dir)(video)

    # -------------------------------------------------------------------------------------
    # load former meta_data
    # with open('/dataset_ssd/ytb_vid_rpn/meta_data.pkl', 'rb') as f:
    #     former_pkl = pickle.load(f)
    # former_dict = {x[0]: x[1] for x in former_pkl if 'ILSVRC2015' in x[0]}
    # former_vid = []
    # for k in former_dict.keys():
    #     former_vid.append((k, former_dict[k]))
    # meta_data.extend(former_vid)
    # -------------------------------------------------------------------------------------

    with Pool(processes=num_threads) as pool:
        for ret in tqdm(pool.imap_unordered(
                functools.partial(worker, output_dir), all_videos), total=len(all_videos)):
            meta_data.append(ret)
            tqdm.write(ret[0])

    # save meta data
    pickle.dump(meta_data, open(os.path.join(output_dir, "meta_data.pkl"), 'wb'))


if __name__ == '__main__':
    Fire(processing)
