import argparse
import time
from sys import platform

from yolov3.models import *
from yolov3.utils.datasets import *
from yolov3.utils.utils import *


class Detect:
    def __init__(self):
        self.cfg = './yolov3/cfg/yolov3.cfg'
        self.data_cfg = './yolov3/data/coco.data'
        self.weights = './yolov3/weights/weights/yolov3.weights'
        self.img_size = 416
        self.conf_thres = 0.5
        self.nms_thres = 0.4
        self.save_txt = False
        self.save_images = True
        self.webcam = True
        # Initialize model
        self.device = torch_utils.select_device()
        if ONNX_EXPORT:
            self.s = (320, 192)  # onnx model image size (height, width)
            self.model = Darknet(self.cfg, self.s)
        else:
            self.model = Darknet(self.cfg, self.img_size)

        # Load weights
        if self.weights.endswith('.pt'):  # pytorch format
            self.model.load_state_dict(torch.load(self.weights, map_location=self.device)['model'])
        else:  # darknet format
            _ = load_darknet_weights(self.model, self.weights)


        # if torch.cuda.device_count() > 1:
        #     self.model = torch.nn.DataParallel(self.model, device_ids=[0, 1])
        # Fuse Conv2d + BatchNorm2d layers
        self.model.fuse()

        # Eval mode
        self.model.to(self.device).eval()
    def detect(self, image):
        self.images = image

        if ONNX_EXPORT:
            img = torch.zeros((1, 3, self.s[0], self.s[1]))
            torch.onnx.export(self.model, img, 'weights/export.onnx', verbose=True)
            return

        # Set Dataloader
        vid_path, vid_writer = None, None
        if self.webcam:
            self.save_images = False
            dataloader = LoadImages(self.images, img_size=self.img_size)
        else:
            dataloader = LoadImages(self.images, img_size=self.img_size)

        # Get classes and colors
        classes = load_classes(parse_data_cfg(self.data_cfg)['names'])
        colors = [[random.randint(0, 255) for _ in range(3)] for _ in range(len(classes))]
        bbox = []
        labels = []
        for i, (img, im0, vid_cap) in enumerate(dataloader):
            t = time.time()
            # save_path = str(Path(self.output) / Path(path).name)

            # Get detections
            img = torch.from_numpy(img).unsqueeze(0).to(self.device)
            pred, _ = self.model(img)
            det = non_max_suppression(pred, self.conf_thres, self.nms_thres)[0]

            if det is not None and len(det) > 0:
                # Rescale boxes from 416 to true image size
                det[:, :4] = scale_coords(img.shape[2:], det[:, :4], im0.shape).round()

                # # print results to screen
                # print('%gx%g ' % img.shape[2:], end='')  # # print image size
                for c in det[:, -1].unique():
                    n = (det[:, -1] == c).sum()
                    # print('%g %ss' % (n, classes[int(c)]), end=', ')

                # Draw bounding boxes and labels of detections
                for *xyxy, conf, cls_conf, cls in det:
                    # if self.save_txt:  # Write to file
                    #     with open(save_path + '.txt', 'a') as file:
                    #         file.write(('%g ' * 6 + '\n') % (*xyxy, cls, conf))

                    # Add bbox to the image
                    label = '%s %.2f' % (classes[int(cls)], conf)
                    plot_one_box(xyxy, im0, label=label, color=colors[int(cls)], line_thickness=1)
                    bbox.append(np.array([(xyxy[0].cpu().detach().numpy()+xyxy[2].cpu().detach().numpy())/2,
                                          (xyxy[1].cpu().detach().numpy()+xyxy[3].cpu().detach().numpy())/2,
                                          (xyxy[2].cpu().detach().numpy()-xyxy[0].cpu().detach().numpy()),
                                          (xyxy[3].cpu().detach().numpy()-xyxy[1].cpu().detach().numpy())]))
                    labels.append(classes[int(cls)])
            # print('Done. (%.3fs)' % (time.time() - t))

            # if self.webcam:  # Show live webcam
            #     cv2.imshow('yolo3', im0)
            #     cv2.waitKey()
            # cv2.imwrite(name, im0)
        # if self.save_images:  # Save image with detections
        #     if dataloader.mode == 'images':
        #         cv2.imwrite(save_path, im0)

        return np.array(bbox),labels






if __name__ == '__main__':
    with torch.no_grad():
        img = '/media/lyc/data/sequences_lt/ballet/color/00000001.jpg'
        img = cv2.imread(img)
        w, h, _ = img.shape
        # img = np.pad(img, ((130, 130), (130, 130), (0, 0)), 'mean')
        # te_im = np.ones((w + 300, h + 300, 3), np.uint8)  # 0 is better than 1 initialization
        # img_mean = np.mean(img, axis=(0,1))
        # te_im[:, :, 0] = img_mean[0]
        # te_im[:, :, 1] = img_mean[1]
        # te_im[:, :, 2] = img_mean[2]
        # te_im[300 // 2:-300 // 2, 300 // 2:-300 // 2, :] = img

        # frame_pad = te_im
        # img = img[420: ,550: ,  :]
        # img[:90, :, :] = 0
        yolo = Detect()
        bbox, label = yolo.detect(img)
        # print(bbox,label)
